use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Database::Transaction;

use Xyzzy::Spiffy -Base;

field 'cfg', -init => 'confess "no configuration"';
field 'txn', -init => 'confess "no transaction"';

sub DESTROY {
	local $@;
	my $cfg = $self->cfg or return;
	my $db = $cfg->{db} or return;
	return unless $db->{Executed};
	warn "rolling back database transaction\n";
	$db->rollback
}

sub AUTOLOAD {
	my $sub = our $AUTOLOAD;
	my $off = rindex($sub, '::');
	confess "no package name in '$sub'"
		if $off == -1;
	my $pkg = substr($sub, 0, $off + 2, '');
	my $txn = $self->txn;
	unless($txn->can($sub)) {
		substr($pkg, -2) = '';
		confess "Can't locate object method \"$sub\" via package \"$pkg\"";
	}
	my $code = "sub $sub { my \$self = shift; return \$self->txn->$sub(\@_) }";
	my $err = do { local $@; eval $code; $@ };
	confess $err if $err;
	return $txn->$sub(@_);
}
